// Enumeration of recognized number literal formats
enum NumberFormat {
    Decimal,
    Hexadecimal,
}

// Result object type returned by parseNumber and complementing functions
struct ParseNumberResult {
    NumberFormat format;
    int value;
    bool error;
}

// Parse a decimal or hexadecimal integer
auto parseNumber(in string src) {
    if(!src.length || src == "0") {
        return ParseNumberResult(NumberFormat.Decimal, 0, false);
    }else if(src.length > 2 && src[0] == '0' && src[1] == 'x') {
        return parseHex(src[2 .. $]);
    }else {
        return parseInt(src);
    }
}

// Parse a decimal integer (with optional sign)
auto parseInt(in string src) {
    assert(src.length);
    const negative = src[0] == '-';
    const hasSign = negative || src[0] == '+';
    int value = 0;
    for(size_t i = hasSign ? 1 : 0; i < src.length; i++) {
        const ch = src[i];
        const oldValue = value;
        if(ch == '_') {
            continue;
        }else if(ch < '0' || ch > '9') {
            return ParseNumberResult(NumberFormat.Decimal, 0, true);
        }else {
            value *= 10;
            value += (ch - '0');
        }
    }
    return ParseNumberResult(
        NumberFormat.Decimal, negative ? -value : value, false
    );
}

// Parse a hexadecimal integer
auto parseHex(in string src) {
    int value = 0;
    for(size_t i = 0; i < 2 * int.sizeof && i < src.length; i++) {
        const char digitChar = src[src.length - i - 1];
        int digitValue = void;
        if(digitChar >= '0' && digitChar <= '9') {
            digitValue = cast(int) (digitChar - '0');
        }else if(digitChar >= 'A' && digitChar <= 'F') {
            digitValue = cast(int) (10 + digitChar - 'A');
        }else if(digitChar >= 'a' && digitChar <= 'f') {
            digitValue = cast(int) (10 + digitChar - 'a');
        }else {
            return ParseNumberResult(NumberFormat.Hexadecimal, 0, true);
        }
        value |= cast(int) (digitValue << (4 * i));
    }
    return ParseNumberResult(
        NumberFormat.Hexadecimal, value, false
    );
}